<?php return array(
    'root' => array(
        'name' => 'grommunio/dav',
        'pretty_version' => '1.0.0+no-version-set',
        'version' => '1.0.0.0',
        'reference' => null,
        'type' => 'library',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => false,
    ),
    'versions' => array(
        'grommunio/dav' => array(
            'pretty_version' => '1.0.0+no-version-set',
            'version' => '1.0.0.0',
            'reference' => null,
            'type' => 'library',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'monolog/monolog' => array(
            'pretty_version' => '2.10.0',
            'version' => '2.10.0.0',
            'reference' => '5cf826f2991858b54d5c3809bee745560a1042a7',
            'type' => 'library',
            'install_path' => __DIR__ . '/../monolog/monolog',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/log' => array(
            'pretty_version' => '1.1.4',
            'version' => '1.1.4.0',
            'reference' => 'd49695b909c3b7628b6289db5479a1c204601f11',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/log',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/log-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0.0 || 2.0.0 || 3.0.0',
            ),
        ),
        'sabre/dav' => array(
            'pretty_version' => '4.4.0',
            'version' => '4.4.0.0',
            'reference' => 'b65362abc926520eda2c57e219f022a6c288069d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/dav',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'sabre/event' => array(
            'pretty_version' => '5.1.7',
            'version' => '5.1.7.0',
            'reference' => '86d57e305c272898ba3c28e9bd3d65d5464587c2',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/event',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'sabre/http' => array(
            'pretty_version' => '5.1.13',
            'version' => '5.1.13.0',
            'reference' => '7c2a14097d1a0de2347dcbdc91a02f38e338f4db',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/http',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'sabre/uri' => array(
            'pretty_version' => '2.3.4',
            'version' => '2.3.4.0',
            'reference' => 'b76524c22de90d80ca73143680a8e77b1266c291',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/uri',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'sabre/vobject' => array(
            'pretty_version' => '4.5.7',
            'version' => '4.5.7.0',
            'reference' => 'ff22611a53782e90c97be0d0bc4a5f98a5c0a12c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/vobject',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'sabre/xml' => array(
            'pretty_version' => '2.2.11',
            'version' => '2.2.11.0',
            'reference' => '01a7927842abf3e10df3d9c2d9b0cc9d813a3fcc',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/xml',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
