<?php

// autoload_classmap.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'BaseException' => UMAPI_PATH . '/class.baseexception.php',
    'BaseExceptionTest' => UMAPI_PATH . '/tests/BaseExceptionTest.php',
    'BaseRecurrence' => UMAPI_PATH . '/class.baserecurrence.php',
    'Composer\\InstalledVersions' => $vendorDir . '/composer/InstalledVersions.php',
    'ExtendedTokenTest' => UMAPI_PATH . '/tests/ExtendedTokenTest.php',
    'FreeBusy' => UMAPI_PATH . '/class.freebusy.php',
    'FreeBusyTest' => UMAPI_PATH . '/tests/FreeBusyTest.php',
    'KeyCloak' => UMAPI_PATH . '/class.keycloak.php',
    'KeyCloakTest' => UMAPI_PATH . '/tests/KeyCloakTest.php',
    'MAPIException' => UMAPI_PATH . '/class.mapiexception.php',
    'MAPIExceptionTest' => UMAPI_PATH . '/tests/MAPIExceptionTest.php',
    'Meetingrequest' => UMAPI_PATH . '/class.meetingrequest.php',
    'Recurrence' => UMAPI_PATH . '/class.recurrence.php',
    'TaskRecurrence' => UMAPI_PATH . '/class.taskrecurrence.php',
    'TaskRequest' => UMAPI_PATH . '/class.taskrequest.php',
    'Token' => UMAPI_PATH . '/class.token.php',
    'TokenTest' => UMAPI_PATH . '/tests/TokenTest.php',
    'UtilityFunctionsTest' => UMAPI_PATH . '/tests/UtilityFunctionsTest.php',
    'resource' => UMAPI_PATH . '/dev/php-mapi-stub.php',
);
